/**
 *
 */
package com.pcloud.book.book.biz.impl;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.pcloud.analysisengine.qrcode.dto.BookTimeSourceDto;
import com.pcloud.appcenter.base.exception.AppBizException;
import com.pcloud.book.base.enums.BookFreezeEnum;
import com.pcloud.book.base.enums.BookTypeEnum;
import com.pcloud.book.base.exception.BookBizException;
import com.pcloud.book.book.biz.BookAdviserBiz;
import com.pcloud.book.book.biz.BookAppBiz;
import com.pcloud.book.book.biz.BookBiz;
import com.pcloud.book.book.biz.BookFreezeBiz;
import com.pcloud.book.book.biz.BookFundBiz;
import com.pcloud.book.book.biz.BookProductBiz;
import com.pcloud.book.book.biz.BookResourceBiz;
import com.pcloud.book.book.cache.BookCache;
import com.pcloud.book.book.constant.BookConstant;
import com.pcloud.book.book.dao.BookDao;
import com.pcloud.book.book.dto.BookAdviserUpdateTimeDTO;
import com.pcloud.book.book.dto.BookAssocCount;
import com.pcloud.book.book.dto.BookCoverImgUpdateDTO;
import com.pcloud.book.book.dto.BookDetialDTO;
import com.pcloud.book.book.dto.BookDto;
import com.pcloud.book.book.dto.BookFreezeDto;
import com.pcloud.book.book.dto.BookFundDto;
import com.pcloud.book.book.entity.Book;
import com.pcloud.book.book.entity.BookAdviser;
import com.pcloud.book.book.set.BookSet;
import com.pcloud.book.consumer.analysisengine.SceneRecordConsr;
import com.pcloud.book.consumer.app.AssistTempletConsr;
import com.pcloud.book.consumer.channel.QrcodeSceneConsr;
import com.pcloud.book.consumer.isbn.IsbnConsr;
import com.pcloud.book.consumer.user.AdviserConsr;
import com.pcloud.book.consumer.user.AgentConsr;
import com.pcloud.book.consumer.user.ChannelConsr;
import com.pcloud.book.consumer.user.PartyConsr;
import com.pcloud.book.mq.producer.BookMQProducer;
import com.pcloud.common.core.aspect.ParamLog;
import com.pcloud.common.core.constant.AgentTypeEnum;
import com.pcloud.common.core.constant.SystemCode;
import com.pcloud.common.exceptions.BizException;
import com.pcloud.common.page.PageBean;
import com.pcloud.common.page.PageBeanNew;
import com.pcloud.common.page.PageParam;
import com.pcloud.common.utils.DateUtils;
import com.pcloud.common.utils.ListUtils;
import com.pcloud.common.utils.string.StringUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;

/**
 * @描述：书籍业务层接口实现类
 * @作者：songx
 * @创建时间：2016年12月23日,下午3:00:08 @版本：1.0
 */
@Service("bookBiz")
public class BookBizImpl implements BookBiz {

	/**
	 *
	 */
	private static final Logger LOGGER = LoggerFactory.getLogger(BookBizImpl.class);
	private static final ExecutorService EXECUTOR_SERVICE = Executors.newFixedThreadPool(5);
	@Autowired
	private BookDao bookDao;
	@Autowired
	private BookFundBiz bookFundBiz;
	@Autowired
	private BookProductBiz bookProductBiz;
	@Autowired
	private BookAppBiz bookAppBiz;
	@Autowired
	private QrcodeSceneConsr qrcodeSceneConsr;
	@Autowired
	private BookAdviserBiz bookAdviserBiz;
	@Autowired
	private BookSet bookSet;
	@Autowired
	private ChannelConsr channelConsr;
	@Autowired
	private IsbnConsr isbnConsr;
	@Autowired
	private BookResourceBiz bookResourceBiz;
	@Autowired
	private BookCache bookCache;
	@Autowired
	private SceneRecordConsr sceneRecordConsr;
	@Autowired
	private AgentConsr agentConsr;
	@Autowired
	private AdviserConsr adviserConsr;
	@Autowired
	private PartyConsr partyConsr;
	@Autowired
	private BookFreezeBiz bookFreezeBiz;
	@Autowired
	private AssistTempletConsr assistTempletConsr;
	@Autowired
	private BookMQProducer bookMQProducer;

	/**
	 * 创建书籍,同时建立与编辑的推广关系
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BookDto create(Book book, String systemCode) throws BizException {
		LOGGER.info("【书籍基础】创建书籍基础信息,<START>.[book]=" + book);
		this.checkAddParam(book);
		// 判断书籍ISBN码是否存在,同时判断书籍ISBN码是否符合规范
		if (this.isbnExists(book.getTypeCode(), book.getIsbn(), book.getSerialNumber())) {
			throw new BookBizException(BookBizException.DB_SELECT_MORE, "书刊已存在");
		}

		bookDao.insert(book);
		BookDto bookDto = bookDao.getByIsbnAndSnum(book.getIsbn(), book.getSerialNumber());
		bookDto.setBookId(book.getBookId());
		bookDto.setTempletId(book.getTempletId());
		// 如果是编辑创建的书籍、默认建立编辑与书籍的推广关系
		if (SystemCode.adviser.code.equalsIgnoreCase(systemCode)) {
			BookAdviser bookAdviser = new BookAdviser();
			bookAdviser.setBookId(bookDto.getBookId());
			bookAdviser.setAdviserId(book.getCreatedUser());
			bookAdviser.setChannelId(book.getChannelId());
			bookAdviser.setCreatedUser(book.getCreatedUser());
			bookAdviser.setTempletId(book.getTempletId());
			bookAdviserBiz.create(bookAdviser);
			// 创建默认数据（二维码及二维码对应数据）
			if (book.getTempletId() != null) {
				Long sceneId = qrcodeSceneConsr.createDefualtByTemplet(book.getTempletId(), book.getChannelId(),
						book.getCreatedUser(), book.getAgentId(), book.getBookId(), null);
				bookDto.setSceneId(sceneId);
			}
			// 二维码不为空的时候,将图书与该二维码关联起来
			if (book.getSceneId() != null) {
				// 关联二维码
				qrcodeSceneConsr.relationQr(book.getBookId(), book.getSceneId(), book.getCreatedUser());
			}
		}
		// 书籍总数缓存加1
		bookCache.incrObject(BookConstant.BOOK_CACHE + "PLATFORM_BOOK_COUNT");
		LOGGER.info("【书籍基础】创建书籍基础信息,<END>");
		return bookDto;
	}

	/**
	 * 校验参数
	 */
	private void checkAddParam(Book book) throws BizException {
		if (StringUtil.isEmpty(book.getIsbn())) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "书籍ISBN码不能为空");
		}
		if (StringUtil.isEmpty(book.getBookName())) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "书籍名称不能为空");
		}
		if (StringUtil.isEmpty(book.getTypeCode())) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "类型编码不能为空");
		}
		if (StringUtil.isEmpty(book.getPublishDate())) {
			book.setPublishDate(null);
		}
		if (StringUtil.isEmpty(book.getSerialNumber())) {
			book.setSerialNumber(null);
		}
	}

	/**
	 * 修改书籍
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public void update(Book book) throws BizException {
		LOGGER.info("【书籍基础】修改书籍基础信息,<PARAM>.[book]=" + book.toString());
		// 校验参数
		this.checkParam(book);
		//获取图书基本信息
		BookDto bookDto = this.getBaseById(book.getBookId());
		checkIsUpdateCoverImgAndSendTopic(bookDto, book.getCoverImg());
		bookDao.update(book);
		// 清除redis中数据
		bookCache.clearRedisByBook(book.getBookId(), book.getIsbn(), book.getSerialNumber());
		LOGGER.info("【书籍基础】修改书籍基础信息,<END>");
	}

	void checkIsUpdateCoverImgAndSendTopic(BookDto oldBook, String newCoverImg){
		if(newCoverImg != null && !newCoverImg.equalsIgnoreCase(oldBook.getCoverImg())){
			BookCoverImgUpdateDTO bookCoverImgUpdateDTO = new BookCoverImgUpdateDTO();
			bookCoverImgUpdateDTO.setBookId(oldBook.getBookId());
			bookCoverImgUpdateDTO.setCoverImg(newCoverImg);
			bookMQProducer.sendUpdateBookCoverImgTopic(bookCoverImgUpdateDTO);
		}
	}

	/**
	 * 修改书籍封面图
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public void updateCoverImg(Book book) throws BizException {
		LOGGER.info("【书籍基础】修改书籍基础信息,<PARAM>.[book]=" + book.toString());
		// 获取图书基本信息校验是否默认图书
		BookDto bookDto = this.getBaseById(book.getBookId());
		if (BookConstant.DEFAULT_BOOK_ISBN.equals(bookDto.getIsbn())
				&& StringUtils.isEmpty(bookDto.getSerialNumber())) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "默认图书不能修改！");
		}
		bookDao.updateCoverImg(book);
		checkIsUpdateCoverImgAndSendTopic(bookDto, book.getCoverImg());
		// 清除redis中数据
		bookCache.clearRedisByBook(bookDto.getBookId(), bookDto.getIsbn(), bookDto.getSerialNumber());
		LOGGER.info("【书籍基础】修改书籍基础信息,<END>");
	}

	/**
	 * 修改书籍(编辑)
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BookDto updateByAdviser(Book book) throws BizException {
		LOGGER.info("【书籍基础】修改书籍基础信息,<PARAM>.[book]=" + book.toString());
		checkParam(book);
		// 设置编辑书刊关联关系
		setBookAdviserRelation(book);
		if (BookConstant.DEFAULT_BOOK_ISBN.equals(book.getIsbn()) && StringUtils.isEmpty(book.getSerialNumber())) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "默认图书不能修改！");
		}
		// 修改书籍
		bookDao.update(book);
		BookDto bookDto = new BookDto();
		bookDto.setBookId(book.getBookId());
		bookDto.setTempletId(book.getTempletId());
		// 创建默认数据（二维码及二维码对应数据）
		if (book.getTempletId() != null) {
			Long sceneId = qrcodeSceneConsr.createDefualtByTemplet(book.getTempletId(), book.getChannelId(),
					book.getCreatedUser(), book.getAgentId(), book.getBookId(), null);
			bookDto.setSceneId(sceneId);
		}

		// 二维码不为空的时候,将图书与该二维码关联起来
		if (book.getSceneId() != null) {
			// 关联二维码
			qrcodeSceneConsr.relationQr(book.getBookId(), book.getSceneId(), book.getCreatedUser());
		}
		// 清除redis中数据
		bookCache.clearRedisByBook(book.getBookId(), book.getIsbn(), book.getSerialNumber());
		checkIsUpdateCoverImgAndSendTopic(bookDto, book.getCoverImg());
		LOGGER.info("【书籍基础】修改书籍基础信息,<END>");
		return bookDto;
	}

	/**
	 * 校验参数
	 */
	private void checkParam(Book book) throws BizException {
		if (StringUtil.isEmpty(book.getIsbn())) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "书籍ISBN码不能为空");
		}
		if (StringUtil.isEmpty(book.getBookName())) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "书籍名称不能为空");
		}
		if (BookTypeEnum.BOOK.value.equalsIgnoreCase(book.getTypeCode())) {
			if (!StringUtils.isEmpty(book.getPublishDate())) {
				try {
					DateUtils.StringToShortDate(book.getPublishDate());
				} catch (Exception e) {
					LOGGER.warn("【书籍-编辑】参数校验:时间格式非法,<ERROR>:" + e.getMessage(), e);
					throw new BookBizException(BookBizException.TIME_FORMAT_ERROR, "出版时间格式非法");
				}
			}
		}
		if (StringUtil.isEmpty(book.getPublishDate())) {
			book.setPublishDate(null);
		}
		if (StringUtil.isEmpty(book.getSerialNumber())) {
			book.setSerialNumber(null);
		}
		if (StringUtil.isEmpty(book.getCoverImg())) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "图书封面图不能为空");
		}
		if (BookConstant.DEFAULT_BOOK_ISBN.equals(book.getIsbn()) && StringUtils.isEmpty(book.getSerialNumber())) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "默认图书不能修改！");
		}
	}

	/**
	 * 删除书籍
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public void delete(Long bookId, Long deleteUserId) throws BizException {
		LOGGER.info("【书籍基础】删除书籍,<PARAM>.[bookId]=" + bookId + "[deleteUserId]=" + deleteUserId);
		if (bookId == null) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "请选择要删除的书刊");
		}
		BookDto book = getBaseById(bookId);
		if (BookConstant.DEFAULT_BOOK_ISBN.equals(book.getIsbn()) && StringUtils.isEmpty(book.getSerialNumber())) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "默认图书不能删除！");
		}
		// 判断是否有基金
		List<BookFundDto> bookFundList = bookFundBiz.getByBookId(bookId);
		if (!ListUtils.isEmpty(bookFundList)) {
			throw new BookBizException(BookBizException.ERROR, "该书刊已经有基金支持，不能删除！");
		}
		// 判断当前和以后图书是否被冻结
		List<BookFreezeDto> bookFreezeList = bookFreezeBiz.getFreezeInfo(bookId);
		if (!ListUtils.isEmpty(bookFreezeList)) {
			throw new BookBizException(BookBizException.ERROR, "该书刊已经被冻结，不能删除！");
		}

		// 删除书籍与编辑关联关系
		bookAdviserBiz.deleteByBook(bookId);
		// 删除图书
		Integer result = bookDao.deleteBook(bookId, deleteUserId);
		if (result == 0) {
			throw BookBizException.DB_DELETE_RESULT_0;
		}
		// 发送书籍删除topic
		bookSet.sendBookDeleteTopic(bookId);
		// 书籍总数缓存减1
		bookCache.decrObject(BookConstant.BOOK_CACHE + "PLATFORM_BOOK_COUNT");
		// 清除缓存
		bookCache.clearRedisByBook(bookId, book.getIsbn(), book.getSerialNumber());
		LOGGER.info("【书籍基础】删除书籍,<END>.[result]=" + result);
	}

	/**
	 * 批量删除书籍
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public void deletes(String bookIdStrs, Long deleteUserId) throws BizException {
		LOGGER.info("【书籍基础】批量删除书籍,<PARAM>.[bookIdStrs]=" + bookIdStrs + "[deleteUserId]=" + deleteUserId);
		if (StringUtil.isEmpty(bookIdStrs)) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "请选择要删除的书刊");
		}
		List<Long> bookIds = Arrays.asList(bookIdStrs.split(",")).stream().map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		if (bookIds == null || bookIds.isEmpty()) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "请选择要删除的书刊");
		}

		// 判断当前和以后基金是否被冻结
		List<BookFreezeDto> bookFreezeList = bookFreezeBiz.listFreezeInfo(bookIds);
		if (!ListUtils.isEmpty(bookFreezeList)) {
			throw new BookBizException(BookBizException.ERROR, "所选书刊中有一部分书刊已经被冻结，不能删除！");
		}

		Integer result = bookDao.deleteByIds(bookIds, deleteUserId);
		if (result == 0) {
			throw BookBizException.DB_DELETE_RESULT_0;
		}

		// 发送书籍批量删除topic
		bookSet.sendBooksDeleteTopic(bookIds);

		// 书籍总数缓存减去相应大小
		Integer bookCount = bookCache.getBookCountToRedis();
		if (null == bookCount || bookCount.equals(0)) {
			bookCount = bookCount - bookIds.size();
		}
		// 清除并将数据存入redis
		bookCache.delBookCountToRedis();
		bookCache.setBookCountToRedis(bookCount);
		LOGGER.info("【书籍基础】批量删除书籍,<END>.[result]=" + result);
	}

	/**
	 * 获取书籍基础信息
	 */
	@Override
	public BookDto getBaseById(Long bookId) throws BizException {
		LOGGER.info("【书籍基础】获取书籍基础信息,<PARAM>.[bookId]=" + bookId);
		if (bookId == null) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "请选择书刊");
		}
		// 从redis获取书籍信息
		BookDto bookDto = bookCache.getBookToRedis(bookId);
		if (null == bookDto) {
			bookDto = bookDao.getBaseById(bookId);
			// 将数据存入redis
			bookCache.setBookToRedis(bookDto);
		}
		return bookDto == null ? new BookDto() : bookDto;
	}

	/**
	 * 调用外部接口新增书籍信息
	 */
	public void insertBook(String isbn) {
		try {
			// 调用外部接口获取该图书的基本信息
			Book bookInfo = isbnConsr.getBook(isbn);
			if (bookInfo != null && bookInfo.getIsbn() != null) {
				// 添加书籍信息(设置添加人为平台)
				bookInfo.setCreatedUser(0l);
				try {
					bookDao.insert(bookInfo);
				} catch (Exception e) {
					LOGGER.error("书刊已存在" + e.getMessage(), e);
				}
			}
		} catch (Exception e) {
			LOGGER.error("ISBN新增书籍失败！" + e.getMessage(), e);
		}
	}

	/**
	 * 根据ISBN获取书刊信息
	 */
	@Override
	public BookDto getByIsbn(String isbn, String serialNumber) throws BizException {
		LOGGER.info("【书刊基础】根据ISBN获取书刊信息,<START>.[isbn]=" + isbn);
		if (StringUtil.isEmpty(isbn)) {
			throw new AppBizException(AppBizException.PARAM_IS_NULL, "请选择书刊");
		}
		if (StringUtils.isEmpty(serialNumber)) {
			serialNumber = null;
		}
		// 从redis获取书刊信息
		BookDto bookDto = bookCache.getBookToRedisByIsbn(isbn, serialNumber);
		if (bookDto == null) {
			bookDto = bookDao.getByIsbnAndSnumContainsDelete(isbn, serialNumber);
			// 将数据存入redis
			bookCache.setBookToRedisByIsbn(bookDto);
		}
		return bookDto;
	}

	/**
	 * 判断ISBN是否存在
	 */
	@Override
	public Boolean isbnExists(String typeCode, String isbn, String serialNumber) throws BizException {
		LOGGER.info("【书籍基础】判断ISBN是否存在,<START>.[isbn]=" + isbn + ",[typeCode]=" + typeCode);
		if (StringUtil.isEmpty(isbn)) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "ISBN码不能为空");
		}
		if (StringUtil.isEmpty(typeCode)) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "书籍类型不能为空");
		}
		if (StringUtil.isEmpty(serialNumber)) {
			serialNumber = null;
		}

		// 校验书号
		bookSet.checkISBN(typeCode, isbn);
		BookDto bookDto = bookDao.getByIsbnAndSnum(isbn, serialNumber);
		if (bookDto == null) {
			return false;
		}
		return true;
	}

	/**
	 * 获取单个书籍基础信息(包含统计信息)
	 */
	@Override
	public BookDto getById(Long bookId, Long adviserId, Long channelId) throws BizException {
		LOGGER.info("【书籍基础】获取单个书籍基础信息(包含统计信息),<START>.[bookId]=" + bookId + ",[adviserId]=" + adviserId
				+ ",[channelId]=" + channelId);
		if (bookId == null) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "请选择书刊");
		}
		if (channelId == null) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "请选择运营平台");
		}
		BookDto bookDto = null;
		Map<String, Object> paramMap = Maps.newHashMap();
		paramMap.put("bookId", bookId);
		paramMap.put("adviserId", adviserId);
		paramMap.put("channelId", channelId);
		bookDto = bookDao.getById(paramMap);
		if (bookDto == null) {
			return new BookDto();
		}
		// 设置渠道信息
		bookDto.setChannelId(channelId);
		bookDto.setChannelName(channelConsr.getNameById(channelId));
		// 设置主编辑信息
		Long mainAdviserId = bookAdviserBiz.getMainAdviserId(bookId, channelId);
		bookSet.setMainEditorInfo(bookDto, mainAdviserId);
		// 设置是否冻结
		bookSet.setFreezeStatus(bookDto);
		// 设置父模板标识
		bookSet.setTempleParentId(bookDto);
		//	设置模板名字
		bookSet.setTemplateName(bookDto);
		// 设置二维码个数
		bookDto.setQrcodeCount(qrcodeSceneConsr.getBookQRCountByMap(adviserId,channelId,bookId));
		LOGGER.info("【书籍基础】获取单个书籍基础信息(包含统计信息),<END>");
		return bookDto;
	}

	/**
	 * 批量获取书籍信息
	 */
	@Override
	public Map<Long, BookDto> getListByIds(List<Long> bookIds) throws BizException {
		LOGGER.info("【书籍基础】批量获取书籍信息,<PARAM>.[bookIds]=" + bookIds.toString());
		if (bookIds == null || bookIds.isEmpty()) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "请选择书刊");
		}
		Map<Long, BookDto> bookMap = null;
		try {
			bookMap = bookDao.getMapByIds(bookIds);
			if (bookMap == null) {
				bookMap = Maps.newHashMap();
			}
		} catch (Exception e) {
			LOGGER.error("【书籍基础】批量获取书籍信息,<ERROR>.[bookDao.getMapByIds]:" + e.getMessage(), e);
			throw BizException.DB_SELECT_IS_FAIL;
		}
		LOGGER.info("【书籍基础】批量获取书籍信息,<END>");
		return bookMap;
	}

	/**
	 * 批量获取书籍信息（书名 与书刊序号 封面图）
	 */
	@Override
	public Map<Long, BookDto> listBaseByIds(List<Long> bookIds) throws BizException {
		LOGGER.info("【书籍基础】批量获取书籍信息（书名 与书刊序号 封面图）,<PARAM>.[bookIds]=" + bookIds.toString());
		if (bookIds == null || bookIds.isEmpty()) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "请选择书刊");
		}
		Map<Long, BookDto> bookMap = null;
		try {
			bookMap = bookDao.listBaseMapByIds(bookIds);
			if (bookMap == null) {
				bookMap = Maps.newHashMap();
			}
		} catch (Exception e) {
			LOGGER.error("【书籍基础】批量获取书籍信息（书名 与书刊序号 封面图）,<ERROR>.[bookDao.listBaseByIds]:" + e.getMessage(), e);
			throw BizException.DB_SELECT_IS_FAIL;
		}
		LOGGER.info("【书籍基础】批量获取书籍信息（书名 与书刊序号 封面图）,<END>");
		return bookMap;
	}

	@Override
	public Map<String, BookDto> listDetailByIds(BookDetialDTO bookDetialDTO) throws BizException {
		Map<String, BookDto> bookMap = null;
		if(null != bookDetialDTO && !(null == bookDetialDTO || ListUtils.isEmpty(bookDetialDTO.getBookIds()) || ListUtils.isEmpty(bookDetialDTO.getChannelId()))){
			Map<String, Object> paramMap = Maps.newHashMap();
			paramMap.put("bookIds",bookDetialDTO.getBookIds());
			paramMap.put("channelIds",bookDetialDTO.getChannelId());
			paramMap.put("adviserId",bookDetialDTO.getAdviserId());
			List<BookDto> bookDtos = bookDao.listDetailByIds(paramMap);
			if(!ListUtils.isEmpty(bookDtos)){
				bookMap = Maps.newHashMap();
				for(BookDto bookDto:bookDtos){
					bookMap.put(bookDto.getChannelId()+"_"+bookDto.getBookId(),bookDto);
				}
			}
		}
		return bookMap;
	}

	/**
	 * 获取书籍列表
	 */
	@Override
	public PageBeanNew<BookDto> getListPage(String isbn, String bookName, String publish, String fundName,
			String serialNumber, Integer isCurrentMonth, Integer currentPage, Integer numPerPage) throws BizException {
		LOGGER.info("【书籍-应用-顾问】获取书籍列表,<START>.");
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("isbn", isbn != null && "".equals(isbn.trim()) ? null : isbn);
		paramMap.put("bookName", bookName != null && "".equals(bookName.trim()) ? null : bookName);
		paramMap.put("publish", publish != null && "".equals(publish.trim()) ? null : publish);
		paramMap.put("fundName", fundName != null && "".equals(fundName.trim()) ? null : fundName);
		paramMap.put("serialNumber", serialNumber != null && "".equals(serialNumber.trim()) ? null : serialNumber);
		paramMap.put("isCurrentMonth", isCurrentMonth);
		paramMap.put("currentPage", currentPage * numPerPage);
		paramMap.put("numPerPage", numPerPage);
		List<BookDto> bookList = bookDao.getListPage4Platform(paramMap);

		// 获取平台端所有账号用户名
		Map<Long, String> pcloudUserMap = partyConsr.getAllUserLoginName(0L, SystemCode.pcloud.code);
		if (!ListUtils.isEmpty(bookList) && !pcloudUserMap.isEmpty()) {
			for (BookDto bookDto : bookList) {
				List<BookFundDto> bookFunds = bookDto.getBookFunds();
				Integer freezeStatus = BookFreezeEnum.WAIT_FREEZE.value;
				Boolean isFundSupport = false;
				BookFreezeDto bf = bookFreezeBiz.getById(bookDto.getBookId());
				String transferor = bf == null ? null : bf.getTransferor();
				if (!ListUtils.isEmpty(bookFunds)) {
					for (BookFundDto bookFundDto : bookFunds) {
						if(StringUtil.isEmpty(transferor)){
							transferor = bookFundDto.getTransferor();
						}
						Date startTime = bookFundDto.getStartTime();
						Date endTime = bookFundDto.getEndTime();
						if (new Date().before(startTime)) {
							bookFundDto.setButton(2);
							freezeStatus = BookFreezeEnum.NO_FREEZE.value;
						} else if (new Date().after(startTime) && new Date().before(endTime)) {
							bookFundDto.setButton(1);
							freezeStatus = BookFreezeEnum.NO_FREEZE.value;
						} else {
							bookFundDto.setButton(0);
						}
						Long opearateUserId = bookFundDto.getLastModifiedUser();
						if (bookFundDto != null && pcloudUserMap.containsKey(opearateUserId)) {
							bookFundDto.setOperateUserName(pcloudUserMap.get(opearateUserId));
						}
						if (new Date().before(endTime)) {
							isFundSupport = true;
						}
					}
					bookDto.setIsHaveFund(true);
				}
				bookDto.setTransferor(transferor);
				bookDto.setIsFundSupport(isFundSupport);
				bookDto.setFreezeStatus(freezeStatus);
			}
		}
		int totalCount = 0;
		if (StringUtil.isEmpty(isbn) && StringUtil.isEmpty(bookName) && StringUtil.isEmpty(publish)
				&& StringUtil.isEmpty(fundName) && StringUtil.isEmpty(serialNumber) && null == isCurrentMonth) {
			totalCount = this.getAllBookCount();
		} else {
			totalCount = bookDao.getBookCount4Platform(paramMap);
		}
		if (!ListUtils.isEmpty(bookList)) {
			bookSet.setPageDetailInfo4Platform(bookList);
		}
		PageBeanNew<BookDto> result = new PageBeanNew<BookDto>();
		result.setCurrentPage(currentPage);
		result.setNumPerPage(numPerPage);
		result.setRecordList(bookList);
		result.setTotalCount(totalCount);
		return result;
	}

	/**
	 * 获取书籍总数信息
	 * 
	 * @return
	 */
	public int getAllBookCount() {
		Integer bookCount = bookCache.getBookCountToRedis();
		if (null == bookCount || bookCount < 10) {
			try {
				bookCount = bookDao.getAllBookCount();
				// 将数据存入redis
				bookCache.setBookCountToRedis(bookCount);
			} catch (Exception e) {
				LOGGER.error("【书籍基础】获取书籍总数信息,<ERROR>.[getAllBookCount]:" + e.getMessage(), e);
				throw BizException.DB_SELECT_IS_FAIL;
			}
		}
		return bookCount;
	}

	/**
	 * 获取书籍列表(编辑)
	 */
	@Override
	public PageBean getListPage4Adviser(Map<String, Object> paramMap, PageParam pageParam, Long adviserId) throws BizException {
		LOGGER.info("【书籍应用】获取书籍列表,<START>.[paramMap]=" + paramMap.toString());
		Long templeId = (Long) paramMap.get("templetId");
		if(null != templeId){
			List<Long> ids = assistTempletConsr.getChildIdListByParentId(templeId);
			if(!ListUtils.isEmpty(ids)){
				paramMap.put("templeIds",ids);
			}
		}
		PageBean pageBean = bookDao.listPage(pageParam, paramMap, "getListPage4Adviser");
		if (pageBean == null || ListUtils.isEmpty(pageBean.getRecordList())) {
			return new PageBean(0, 0, new ArrayList<>());
		}
		// 填充渠道基础信息
		bookSet.setChannelInfoList(pageBean.getRecordList());
		bookSet.setBookSvUvPv4Object(pageBean.getRecordList());
		// 填充模板信息
		bookSet.setTemplateName(pageBean.getRecordList());
		// 设置是否被冻结
		bookSet.setFreezeStatus(pageBean.getRecordList());
		// 设置图书是否关联二维码
		bookSet.setIsRelationQrcodeAndCount(pageBean.getRecordList(),adviserId);
		//设置图书收益
		bookSet.setBookIncome(pageBean.getRecordList());
		// 设置二维码相关信息
		bookSet.setQrInfo(pageBean.getRecordList());
		LOGGER.info("【书籍应用】获取书籍列表,<END>");
		return pageBean;
	}

	@ParamLog(description = "首页展示正在做的图书")
	@Override
	public List<BookDto> getList4Adviser(Long adviserId, Integer num) throws BizException {
		Map<String,Object> paramMap = Maps.newHashMap();
		paramMap.put("adviserId",adviserId);
		paramMap.put("num",num);
		paramMap.put("isbns",BookConstant.DEFAULT_BOOK_ISBN);
		List<BookDto> bookDtos = bookDao.getList4Adviser(paramMap);
		if(!ListUtils.isEmpty(bookDtos)){
			// 填充渠道基础信息
			bookSet.setChannelInfoList(Lists.newArrayList(bookDtos));
			bookSet.setBookSvUvPv4Object(Lists.newArrayList(bookDtos));
			// 填充模板信息
			bookSet.setTemplateName(Lists.newArrayList(bookDtos));
			// 设置是否被冻结
			bookSet.setFreezeStatus(Lists.newArrayList(bookDtos));
			// 设置图书是否关联二维码
			bookSet.setIsRelationQrcodeAndCount(Lists.newArrayList(bookDtos),adviserId);
			//设置图书收益
			bookSet.setBookIncome(Lists.newArrayList(bookDtos));
			// 设置二维码相关信息
			bookSet.setQrInfo(Lists.newArrayList(bookDtos));
		}
		return bookDtos;
	}

	/**
	 * 微信端获取书籍列表(编辑)
	 */
	@Override
	public PageBean getListPage4Wechat(Map<String, Object> paramMap, PageParam pageParam, Long adviserId)
			throws BizException {
		LOGGER.info("【书籍应用】微信端获取书籍列表,<START>.[paramMap]=" + paramMap.toString());
		PageBean pageBean = bookDao.listPage(pageParam, paramMap, "getListPage4Adviser");
		if (pageBean == null || ListUtils.isEmpty(pageBean.getRecordList())) {
			return new PageBean(0, 0, new ArrayList<>());
		}
		// 填充渠道基础信息
		bookSet.setPageDetailInfo4Wechat(pageBean.getRecordList(), adviserId);
		// 设置图书扫码量读者量浏览量
		bookSet.setBookSvUvPv4Object(pageBean.getRecordList());
		// 填充模板信息
		bookSet.setTemplateName(pageBean.getRecordList());
		// 设置冻结状态
		bookSet.setFreezeStatus(pageBean.getRecordList());
		LOGGER.info("【书籍应用】微信端获取书籍列表,<END>");
		return pageBean;
	}

	/**
	 * 获取书籍详细信息
	 */
	@Override
	public BookDto getDetailById(Long bookId, Long adviserId, Long channelId) throws BizException {
		LOGGER.info("【书籍应用】获取书籍详细信息,<START>.[bookId]=" + bookId + ",[channelId]=" + channelId + ",[adviserId]="
				+ adviserId);
		if (bookId == null) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "请选择书刊");
		}
		if (channelId == null) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "请选择运营平台");
		}
		// 如果编辑标识为空，取配置该书籍的编辑的top1
		if (adviserId == null) {
			adviserId = bookAdviserBiz.getAdviserTop1(bookId, channelId);
		}
		// 获取书籍基本信息
		BookDto bookDto = this.getBaseById(bookId);
		if (BookTypeEnum.JOURNAL.value.equals(bookDto.getTypeCode())) {
			// 获取运营名称
			String channelName = channelConsr.getNameById(channelId);
			bookDto.setChannelName(channelName);
			bookDto.setAgentType(agentConsr.getAgentType(channelId, SystemCode.channel.code));
		}
		// 填充书籍关联的问答
		bookDto.setProducts(bookProductBiz.getQaListById(bookId, adviserId, channelId));
		LOGGER.info("【书籍应用】获取书籍详细信息,<END>");
		return bookDto;
	}

	/**
	 * 根据名称获取ID
	 */
	@Override
	public List<Long> getIdsByName(String bookName) throws BizException {
		LOGGER.info("【书籍顾问】根据名称获取ID,<START>.[bookName]=" + bookName);
		if (StringUtil.isEmpty(bookName)) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "请输入书籍名称");
		}
		List<Long> bookIds = null;
		try {
			bookIds = bookDao.getIdsByName(bookName);
			if (bookIds == null) {
				bookIds = Lists.newArrayList();
			}
		} catch (Exception e) {
			LOGGER.error("【书籍-顾问】根据名称获取ID,<ERROR>.[getIdsByName]:" + e.getMessage(), e);
			throw BizException.DB_SELECT_IS_FAIL;
		}
		LOGGER.info("【书籍顾问】根据名称获取ID,<END>");
		return bookIds;
	}
	
	/**
	 * 根据名称获取ID
	 */
	@Override
	public List<Long> getIdsByName(String bookName, Long adviserId) throws BizException {
		LOGGER.info("【书籍顾问】根据名称获取ID,<START>.[bookName|adviserId]=" + bookName+"|"+adviserId);
		if (StringUtil.isEmpty(bookName)) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "请输入书籍名称");
		}
		if (adviserId == null) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "请输入编辑ID");
		}
		List<Long> bookIds = null;
		try {
			Map<String, Object> paramMap = new HashMap<String, Object>();
			paramMap.put("bookName", bookName);
			paramMap.put("adviserId", adviserId);
			bookIds = bookDao.getIdsByNameMap(paramMap);
			if (bookIds == null) {
				bookIds = Lists.newArrayList();
			}
		} catch (Exception e) {
			LOGGER.error("【书籍-顾问】根据名称获取ID,<ERROR>.[getIdsByNameMap]:" + e.getMessage(), e);
			throw BizException.DB_SELECT_IS_FAIL;
		}
		LOGGER.info("【书籍顾问】根据名称获取ID,<END>");
		return bookIds;
	}

	/**
	 * 图书收益，获取书籍列表(编辑)
	 */
	@Override
	public PageBean getListPage4AdviserProfit(Map<String, Object> paramMap, PageParam pageParam) throws BizException {
		LOGGER.info("【书籍收益】获取书籍列表,<START>.[START]=" + paramMap.toString());
		try {
			return bookDao.listPage(pageParam, paramMap, "getList4AdviserProfit");
		} catch (Exception e) {
			LOGGER.error("【书籍收益】获取书籍列表,<ERROR>.[getList4AdviserProfit]:" + e.getMessage(), e);
			throw BizException.DB_SELECT_IS_FAIL;
		}
	}

	/**
	 * 图书收益，获取单个书籍基础信息(包含统计信息)
	 */
	@Override
	public BookDto getById4Profit(Long bookId, Long adviserId, Long channelId) throws BizException {
		LOGGER.info("【图书收益】获取单个书籍基础信息(包含统计信息),<PARAM>.[bookId]=" + bookId + ",[adviserId]=" + adviserId
				+ ",[channelId]=" + channelId);
		if (bookId == null) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "请选择书刊");
		}
		if (channelId == null) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "请选择运营平台");
		}
		BookDto bookDto = this.getBaseById(bookId);
		if (bookDto.getBookId() == null) {
			return bookDto;
		}
		// 获取第一次扫描时间
		BookTimeSourceDto bookTimeSourceDto = sceneRecordConsr.getBookScanInfo(bookId, channelId, adviserId);
		if (bookTimeSourceDto != null) {
			bookDto.setFirstScanDate(bookTimeSourceDto.getScanTimes());
			bookDto.setFirstSceneName(bookTimeSourceDto.getSceneName());
		}
		// 设置渠道名称
		bookDto.setChannelName(channelConsr.getNameById(channelId));
		LOGGER.info("【图书收益】获取单个书籍基础信息(包含统计信息),<END>");
		return bookDto;
	}

	/**
	 * 判断isbn是否存在,存在返回书籍信息
	 *
	 * @throws IOException
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public BookDto isbnExistsOrGetInfo(Book book) {
		LOGGER.info("【书籍基础】判断ISBN是否存在,<PARAM>.[isbn]=" + book.getIsbn());
		// 校验书号合法性
		bookSet.checkISBN(book.getTypeCode(), book.getIsbn());
		if (StringUtils.isEmpty(book.getSerialNumber())) {
			book.setSerialNumber(null);
		}
		// 获取最近书刊信息
		BookDto journalDto = bookDao.getByIsbn(book.getIsbn());
		if (BookTypeEnum.BOOK.value.equals(book.getTypeCode()) && journalDto == null) {
			try {
				// 调用外部接口查询书籍信息
				insertBook(book.getIsbn());
				journalDto = bookDao.getByIsbn(book.getIsbn());
			} catch (Exception e) {
				LOGGER.error("【书刊基础】根据ISBN获取书刊信息,<ERROR>.[getByJournIsbn]:" + e.getMessage(), e);
				throw BizException.DB_SELECT_IS_FAIL;
			}
		}

		if (null != journalDto) {
			BookDto journal = bookDao.getByIsbnAndSnum(book.getIsbn(), book.getSerialNumber());
			if (journal != null) {
				journalDto = journal;
			}
		}

		return journalDto;
	}

	/**
	 * 创建书籍顾问关联关系
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public void setBookAdviserRelation(Book book) throws BizException {
		BookAdviser bookAdviser = new BookAdviser();
		bookAdviser.setBookId(book.getBookId());
		bookAdviser.setAdviserId(book.getCreatedUser());
		bookAdviser.setChannelId(book.getChannelId());
		bookAdviser.setCreatedUser(book.getCreatedUser());
		bookAdviser.setSerialNumber(book.getSerialNumber());// 设置刊物序号
		bookAdviser.setTempletId(book.getTempletId());
		bookAdviserBiz.create(bookAdviser);
	}

	/**
	 * 获取书籍基本信息
	 */
	@Override
	public BookDto getBaseInfoById4Wechat(Long bookId) {
		LOGGER.info("【书籍-应用-微信】获取书籍基本信息,<START>.[bookId]=" + bookId);
		if (bookId == null) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "请选择书刊");
		}
		// 获取书籍基本信息
		BookDto bookDto = this.getBaseById(bookId);
		LOGGER.info("【书籍-应用-微信】获取书籍基本信息,<END>");
		return bookDto;
	}

	/**
	 * 根据ISBN获取期刊分页列表
	 */
	@Override
	public PageBean getJournalPageByIsbn(PageParam pageParam, String typeCode, String isbn, Long channelId,
			Long wechatUserId) {
		LOGGER.info("【书籍-书刊管理-微信】根据ISBN获取期刊分页列表getJournalPageByIsbn,<START>.[isbn]=" + isbn);
		if (isbn == null || "".equals(isbn.toLowerCase().trim())) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "isbn不存在!");
		}
		Map<String, Object> paramMap = Maps.newHashMap();
		paramMap.put("typeCode", typeCode);
		paramMap.put("isbn", isbn);
		paramMap.put("channelId", channelId);
		PageBean pageBean = bookDao.listPage(pageParam, paramMap, "getJournalPage4Adviser");
		if (pageBean == null || ListUtils.isEmpty(pageBean.getRecordList())) {
			return new PageBean();
		}
		// 设置是否加入书架
		bookSet.setIsHaveAddBookcase(pageBean.getRecordList(), wechatUserId);
		// 设置模板名称
		bookSet.setTemplateName(pageBean.getRecordList());
		LOGGER.info("【书籍-书刊管理-微信】根据ISBN获取期刊分页列表getJournalPageByIsbn,<END>.[isbn]=" + isbn);
		return pageBean;
	}

	/**
	 * 获取书刊列表（微信客户端）
	 */
	@Override
	public PageBean listPage4Wechat(PageParam pageParam, Long channelId, Long wechatUserId) {
		LOGGER.info("【书籍-书刊管理-微信】分页获取书刊列表,<START>.[channelId]=" + channelId);
		Map<String, Object> paramMap = Maps.newHashMap();
		paramMap.put("channelId", channelId);
		PageBean pageBean = bookDao.listPage(pageParam, paramMap, "listPage4Wechat");
		if (pageBean == null || pageBean.getRecordList().isEmpty()) {
			return new PageBean();
		}
		// 设置是否加入书架
		bookSet.setIsHaveAddBookcase(pageBean.getRecordList(), wechatUserId);
		// 设置图书扫码量读者量浏览量
		bookSet.setBookSvUvPv4Object(pageBean.getRecordList());
		// 设置模板名称(图书类型)
		bookSet.setTemplateName(pageBean.getRecordList());
		return pageBean;
	}

	/**
	 * 根据ISBN码获取书籍详情（包含二维码个数，应用个数，商品个数）
	 */
	@Override
	public BookDto getDetailByIsbn(String isbn, String serialNumber) {
		LOGGER.info("根据ISBN码获取书籍详情（包含二维码个数，应用个数，商品个数）,isbn=" + isbn);
		Map<String, Object> paramMap = new HashMap<String, Object>();
		paramMap.put("isbn", isbn);
		paramMap.put("serialNumber", serialNumber);
		// 获取书籍基本信息
		BookDto bookDto = bookDao.getDetailByIsbn(paramMap);
		if (bookDto != null) {
			Map<String, Object> qrMap = qrcodeSceneConsr.getQr4Book(bookDto.getBookId());
			if (qrMap != null) {
				bookDto.setQrcodeCount((Integer) qrMap.get("qrCount"));
				bookDto.setLatestQrcode((String) qrMap.get("qrcodeUrl"));
			}
		}
		return bookDto;
	}

	/**
	 * 获取书刊信息
	 */
	@Override
	public List<Object> listBook(Long adviserId, Integer number) {
		LOGGER.info("【书籍-首页】 获取书刊信息,<START>.[adviserId]=" + adviserId + "number" + number);
		Map<String, Object> paramMap = new HashMap<>();
		paramMap.put("adviserId", adviserId);
		paramMap.put("number", number);
		List<Object> books = bookDao.listBy(paramMap, "listBook");
		// 填充渠道基础信息
		bookSet.setChannelInfoList(books);
		// 设置图书扫码量读者量浏览量
		bookSet.setBookSvUvPv4Object(books);
		// 设置模板名称(图书类型)
		bookSet.setTemplateName(books);
		LOGGER.info("【书籍-首页】获取书籍列表,<END>");
		return books;
	}

	/**
	 * 获取书刊信息 app - 编辑
	 */
	@Override
	public PageBean listBook4App(Long adviserId, PageParam pageParam, String bookName) {
		LOGGER.info("【书籍-首页】 获取书刊信息,<START>.[adviserId]=" + adviserId + "pageParam" + pageParam);
		Map<String, Object> paramMap = new HashMap<>();
		paramMap.put("adviserId", adviserId);
		paramMap.put("bookName", bookName);
		PageBean pageBean = null;
		try {
			pageBean = bookDao.listPage(pageParam, paramMap, "listBook4App");
		} catch (Exception e) {
			LOGGER.error("【书籍-首页】获取书籍列表,<ERROR>.[listPage]:" + e.getMessage(), e);
			throw BizException.DB_SELECT_IS_FAIL;
		}
		if (pageBean == null || ListUtils.isEmpty(pageBean.getRecordList())) {
			return new PageBean();
		}
		// 设置模板名称(图书类型)
		bookSet.setTemplateName(pageBean.getRecordList());
		// 设置图书扫码量读者量浏览量
		bookSet.setBookSvUvPv4Object(pageBean.getRecordList());
		// 设置基金状态
		bookSet.setFreezeStatus(pageBean.getRecordList());
		LOGGER.info("【书籍-首页】获取书籍列表,<END>");
		return pageBean;
	}

	/**
	 * 书刊列表-出版端
	 */
	@Override
	public PageBean listPage4Agent(PageParam pageParam, Map<String, Object> paramMap, Long agentId) {
		LOGGER.info("书刊列表-出版端,<START>.[paramMap]=" + paramMap + ",agentId" + agentId);
		List<Long> adviserIds = adviserConsr.getIdsByNameAndAgentId(agentId, (String) paramMap.get("mainEditorName"));
		paramMap.put("adviserIds", adviserIds);
		if (ListUtils.isEmpty(adviserIds)) {
			return new PageBean(0, 0, new ArrayList<>());
		}
		//图书类型查询
		Long templeId = (Long) paramMap.get("templetId");
		if(null != templeId){
			List<Long> ids = assistTempletConsr.getChildIdListByParentId(templeId);
			if(!ListUtils.isEmpty(ids)){
				paramMap.put("templeIds",ids);
			}
		}
		PageBean pageBean = bookDao.listPage(pageParam, paramMap, "listPage4Agent");
		if (pageBean == null || ListUtils.isEmpty(pageBean.getRecordList())) {
			return new PageBean();
		}
		// 填充渠道基础信息
		bookSet.setChannelInfoList(pageBean.getRecordList());
		// 设置编辑名称
		bookSet.setAdviserName4BookDto(pageBean.getRecordList());
		// 设置图书扫码量读者量浏览量
		bookSet.setBookSvUvPv4Object(pageBean.getRecordList());
		// 设置冻结状态
		bookSet.setFreezeStatus(pageBean.getRecordList());
		// 设置模板名称(图书类型)
		bookSet.setTemplateName(pageBean.getRecordList());
		if(agentId.equals(getAgentIdByType())){
			//	设置待审核二维码个数
			bookSet.setAuditWaitNum(pageBean.getRecordList());
		}
		LOGGER.info("书刊列表-出版端,<END>");
		return pageBean;
	}

	/**
	 * 更新图书模板
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Long updateBookTemplet(Book book, Long adviserId, Long agentId) {
		LOGGER.info("书刊-更新图书模板,<START>.[ book]=" + book);
		if (book.getBookId() == null || book.getTempletId() == null || book.getChannelId() == null) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "参数缺失,请稍后重试");
		}
		bookAdviserBiz.updateBookTemplet(book.getBookId(), book.getChannelId(), adviserId, book.getTempletId());
		Long sceneId = qrcodeSceneConsr.createDefualtByTemplet(book.getTempletId(), book.getChannelId(), adviserId,
				agentId, book.getBookId(), book.getSceneId());
		return sceneId;
	}

	/**
	 * 获取应用关联图书
	 */
	@Override
	public List<BookDto> listBookByAppId(Long appId, Long channelId) {
		LOGGER.info("【编辑书刊】获取应用关联图书,<START>.[appId]=" + appId + ",[channelId]=" + channelId);
		try {
			Map<String, Object> paramMap = Maps.newHashMap();
			paramMap.put("appId", appId);
			paramMap.put("channelId", channelId);
			return bookDao.listBookByAppId(paramMap);
		} catch (Exception e) {
			LOGGER.error("【编辑书刊】获取应用关联图书,<ERROR>.[listBookByAppId]:" + e.getMessage(), e);
			throw new BookBizException(BookBizException.DB_DML_FAIL, "获取应用关联图书失败");
		}
	}

	/**
	 * 书刊回收站列表（30天内删除的书刊列表）
	 */
	@Override
	public PageBean listPageDelete4Adviser(Long adviserId, PageParam pageParam) {
		LOGGER.info("【编辑书刊】书刊回收站列表,<START>.[adviserId]=" + adviserId);
		PageBean pageBean = null;
		Map<String, Object> paramMap = new HashMap<>();
		paramMap.put("adviserId", adviserId);
		try {
			pageBean = bookDao.listPage(pageParam, paramMap, "listPageDelete4Adviser");
		} catch (Exception e) {
			LOGGER.error("【编辑书刊】书刊回收站列表,<ERROR>.[listPageDelete4Adviser]:" + e.getMessage(), e);
			throw BizException.DB_SELECT_IS_FAIL;
		}
		// 填充渠道基础信息
		bookSet.setChannelInfoList(pageBean.getRecordList());
		// 设置图书扫码量读者量浏览量
		bookSet.setBookSvUvPv4Object(pageBean.getRecordList());
		// 设置图书类型
		bookSet.setTemplateName(pageBean.getRecordList());
		LOGGER.info("【编辑书刊】书刊回收站列表,<END>");
		return pageBean;
	}

	/**
	 * 获取图书关联统计个数(商品，应用，资源)
	 */
	@Override
	public BookAssocCount getCount4BookAssoc(Long bookId, Long channelId, Long adviserId) {
		LOGGER.info("【编辑书刊】获取图书关联统计个数(商品，应用，资源),<START>.[bookId]=" + bookId + ",channelId=" + channelId);
		BookAssocCount bookAssocCount = new BookAssocCount();
		bookAssocCount.setResourceCount(bookResourceBiz.getResourceCount(adviserId, bookId, channelId));
		bookAssocCount.setProductCount(bookProductBiz.getProCount(adviserId, bookId, channelId));
		bookAssocCount.setSuperProCount(bookProductBiz.getSuperProCount(adviserId, bookId, channelId));
		bookAssocCount.setAppCount(bookAppBiz.getAppCount(adviserId, bookId, channelId));
		LOGGER.info("【编辑书刊】书刊回收站列表<END>");
		return bookAssocCount;
	}

	/**
	 * 导出出版下所有书刊excel表（出版端）
	 */
	@Override
	public void exportBookToExcel(Long agentId) {
		LOGGER.info("导出出版下所有书刊excel表（出版端），agentId=" + agentId);
		// 获取代理下所有编辑
		List<Long> adviserIds = adviserConsr.getByAgentId(agentId);
		if (ListUtils.isEmpty(adviserIds)) {
			throw new BookBizException(BookBizException.ERROR, "您还没有图书！");
		}
		EXECUTOR_SERVICE.execute(new Runnable() {
			public void run() {
				Date sendDate = new Date();
				try {
					// 获取出版下所有二维码
					Map<String, Object> param = new HashMap<String, Object>();
					param.put("adviserIds", adviserIds);
					// 获取默认书籍
					BookDto book = getByIsbn("9787507600000", null);
					if (book != null) {
						param.put("bookId", book.getBookId());
					}
					// 获取图书总数除RAYS现代纸书
					Integer count = bookDao.getAllBookCount4Agent(param);
					List<Object[]> dataList = new ArrayList<Object[]>();
					if (count > BookConstant.BOOK_SELECT_COUNT) {
						for (int i = 0, titalCount = (count + BookConstant.BOOK_SELECT_COUNT - 1)
								/ BookConstant.BOOK_SELECT_COUNT; i < titalCount; i++) {
							param.put("currentPage", i * BookConstant.BOOK_SELECT_COUNT);
							param.put("numPerPage", BookConstant.BOOK_SELECT_COUNT);
							List<Object> agentBook = bookDao.listBy(param, "getAllBook4Agent");
							List<Object[]> excelData = bookSet.getExcelData(agentBook);
							dataList.addAll(excelData);
							agentBook.clear();
							excelData.clear();
						}
					} else {
						List<Object> agentBook = bookDao.listBy(param, "getAllBook4Agent");
						List<Object[]> excelData = bookSet.getExcelData(agentBook);
						dataList.addAll(excelData);
					}
					// 生成excel
					String url = bookSet.createExcel(dataList);
					if (!StringUtils.isEmpty(url)) {
						// 发送站内信
						bookSet.sendBookExportLetter(agentId, url, sendDate);
					}
				} catch (Exception e) {
					LOGGER.error("生成导出文件失败", e);
				}
			}
		});

	}

	/**
	 * 根据isbn获取图书信息
	 */
	@Override
	public Map<Long, BookDto> listByIsbn(String isbn) {
		LOGGER.info("根据isbn获取图书信息isbn=" + isbn);
		Map<Long, BookDto> bookMap = bookDao.listByIsbn(isbn);
		LOGGER.info("根据isbn获取图书信息END");
		return bookMap;
	}

	/**
	 * 图书列表（平台端）
	 */
	@Override
	public PageBean listBookClassify(PageParam pageParam, Map<String, Object> paramMap) {
		LOGGER.info("图书列表（平台端）<START>,paramMap=" + paramMap);
		Long templeId = (Long) paramMap.get("templetId");
		if(null != templeId){
			List<Long> ids = assistTempletConsr.getChildIdListByParentId(templeId);
			if(!ListUtils.isEmpty(ids)){
				paramMap.put("templeIds",ids);
			}
		}
		PageBean pageBean = bookDao.listPage(pageParam, paramMap, "listBookClassify");
		if (pageBean != null) {
			// 设置编辑名称
			bookSet.setAdviserName4BookDto(pageBean.getRecordList());
			// 设置图书读者量、扫码量、浏览量
			bookSet.setBookSvUvPv4Object(pageBean.getRecordList());
			// 设置书刊分类
			bookSet.setTemplateName(pageBean.getRecordList());
			// 设置运营平台名称
			bookSet.setChannelInfoList(pageBean.getRecordList());
			// 设置图书收益
			bookSet.setBookIncome(pageBean.getRecordList());
		} else {
			pageBean = new PageBean();
		}
		LOGGER.info("图书列表（平台端）<END>");
		return pageBean;
	}

	/**
	 * 导出平台下所有书刊excel表（平台端）
	 */
	@Override
	public void exportBookToExcel4Platform() {
		LOGGER.info("导出平台下所有书刊excel表（平台端）<START>");

		List<Object> platformBook = bookDao.listBy(null, "getAllBook4Platform");
		if (ListUtils.isEmpty(platformBook)) {
			throw new BookBizException(BookBizException.ERROR, "您还没有图书！");
		}

		EXECUTOR_SERVICE.execute(new Runnable() {
			public void run() {
				Date sendDate = new Date();
				try {
					// 获取出版下所有二维码
					Map<String, Object> param = new HashMap<String, Object>();
					// 获取默认书籍
					BookDto book = getByIsbn("9787507600000", null);
					if (book != null) {
						param.put("bookId", book.getBookId());
					}
					// 获取图书总数除RAYS现代纸书
					Integer count = bookDao.getAllBookCount4Platform(param);
					List<Object[]> dataList = new ArrayList<Object[]>();
					if (count > BookConstant.BOOK_SELECT_COUNT) {
						for (int i = 0, titalCount = (count + BookConstant.BOOK_SELECT_COUNT - 1)
								/ BookConstant.BOOK_SELECT_COUNT; i < titalCount; i++) {
							param.put("currentPage", i * BookConstant.BOOK_SELECT_COUNT);
							param.put("numPerPage", BookConstant.BOOK_SELECT_COUNT);
							List<Object> partPlatformBook = bookDao.listBy(param, "getAllBook4Platform");

							// 设置编辑名称
							bookSet.setAdviserName4BookDto(partPlatformBook);
							// 设置图书读者量、扫码量、浏览量
							bookSet.setBookSvUvPv4Object(partPlatformBook);
							// 设置书刊分类
							bookSet.setTemplateName(partPlatformBook);
							// 设置运营平台名称
							bookSet.setChannelInfoList(partPlatformBook);
							// 设置图书收益
							bookSet.setBookIncome(partPlatformBook);

							List<Object[]> excelData = bookSet.getExcelData4Platform(partPlatformBook);
							dataList.addAll(excelData);
							partPlatformBook.clear();
							excelData.clear();
						}
					} else {
						List<Object> partPlatformBook = bookDao.listBy(param, "getAllBook4Platform");

						// 设置编辑名称
						bookSet.setAdviserName4BookDto(partPlatformBook);
						// 设置图书读者量、扫码量、浏览量
						bookSet.setBookSvUvPv4Object(partPlatformBook);
						// 设置书刊分类
						bookSet.setTemplateName(partPlatformBook);
						// 设置运营平台名称
						bookSet.setChannelInfoList(partPlatformBook);
						// 设置图书收益
						bookSet.setBookIncome(partPlatformBook);

						List<Object[]> excelData = bookSet.getExcelData4Platform(partPlatformBook);
						dataList.addAll(excelData);
					}
					// 生成excel
					String url = bookSet.createExcel(dataList);
					if (!StringUtils.isEmpty(url)) {
						// 发送站内信
						bookSet.sendBookExportLetter4Platform(-1L, url, sendDate);
					}
				} catch (Exception e) {
					LOGGER.error("生成导出文件失败", e);
				}
			}
		});

		LOGGER.info("导出平台下所有书刊excel表（平台端）<END>");
	}



	/**
	 * 获取图书基本信息（编辑 - 运营 - 图书）
	 */
	@Override
	public BookDto getBookById(Long adviserId, Long channelId, Long bookId) {
		LOGGER.info(
				"获取图书基本信息（编辑 - 运营 - 图书）<START>adviserId=" + adviserId + "channelId=" + channelId + "bookId=" + bookId);
		BookDto bookDto = this.getBaseById(bookId);
		if (bookDto != null) {
			// 获取图书类型
			Long templetId = bookDao.getTempletId(adviserId, channelId, bookId);
			bookDto.setTempletId(templetId);
			// 设置图书类型名称
			bookSet.setTemplateName(bookDto);
		}
		LOGGER.info("获取图书基本信息（编辑 - 运营 - 图书）<END>");
		return bookDto;
	}

	/**
	 * 根据ISBN获取书刊信息
	 */
	@Override
	public BookDto getByIsbnAndTypeCode(String isbn, String typeCode) throws BizException {
		LOGGER.info("【书刊基础】根据ISBN获取书刊信息,<START>.[isbn]=" + isbn + "typeCode=" + typeCode);
		if (StringUtil.isEmpty(isbn)) {
			throw new AppBizException(AppBizException.PARAM_IS_NULL, "请选择书刊");
		}
		BookDto bookDto = bookDao.getByIsbn(isbn);
	/*	if (BookTypeEnum.BOOK.value.equals(typeCode) && bookDto == null) {
			// 调用外部接口查询书籍信息
			insertBook(isbn);
			bookDto = bookDao.getByIsbn(isbn);
		}*/
		LOGGER.info("【书刊基础】根据ISBN获取书刊信息,<END>.[bookDto]=" + bookDto);
		return bookDto;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public BookDto create4Adviser(Book book) {
		LOGGER.info("编辑新增图书book=" + book);
		this.checkAddParam(book);
		if (book.getChannelId() == null) {
			throw new BookBizException(BookBizException.PARAM_IS_NULL, "请选择运营平台");
		}
		// 校验书号
		bookSet.checkISBN(book.getTypeCode(), book.getIsbn());
		BookDto bookDto = bookDao.getByIsbnAndSnum(book.getIsbn(), book.getSerialNumber());
		if (bookDto == null) {
			// 图书不存在，新增图书
			bookDao.insert(book);
		} else {
			// 图书已存在
			book.setLastModifiedUser(bookDto.getAdviserId());
			book.setBookId(bookDto.getBookId());
		}
		// 新增图书与编辑关联关系
		createBookAdviserRelation(book);
		// 图书不存在，新增图书，图书数量加1
		if (bookDto == null) {
			// 书籍总数缓存加1
			bookCache.incrObject(BookConstant.BOOK_CACHE + "PLATFORM_BOOK_COUNT");
		} else {
			checkIsUpdateCoverImgAndSendTopic(bookDto, book.getCoverImg());
			// 修改图书基本信息
			bookDao.updateByAdviser(book);
			// 清除redis中数据
			bookCache.clearRedisByBook(book.getBookId(), book.getIsbn(), book.getSerialNumber());
		}
		bookDto = bookDao.getByIsbnAndSnum(book.getIsbn(), book.getSerialNumber());
		bookDto.setChannelId(book.getChannelId());
		LOGGER.info("【书籍基础】创建书籍基础信息,<END>");
		return bookDto;
	}

    /**
     * 修改图书与图书类型
     * @param book
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBookAndBookType(Book book) {
        LOGGER.info("修改图书基本信息与类型book=" + book);
        this.checkParam(book);
		//获取图书基本信息
		BookDto bookDto = this.getBaseById(book.getBookId());
		checkIsUpdateCoverImgAndSendTopic(bookDto, book.getCoverImg());
		// 修改图书基本信息
		bookDao.updateByAdviser(book);
        // 清除redis中数据
        bookCache.clearRedisByBook(book.getBookId(), book.getIsbn(), book.getSerialNumber());
        //修改图书类型
        bookAdviserBiz.updateBookTemplet(book.getBookId(),book.getChannelId(),book.getLastModifiedUser(),book.getTempletId());
        LOGGER.info("修改图书基本信息与类型[END]");
	}

    /**
	 * 新增图书与编辑关联关系
	 * 
	 * @param book
	 */
	void createBookAdviserRelation(Book book) {
		BookAdviser bookAdviser = new BookAdviser();
		bookAdviser.setBookId(book.getBookId());
		bookAdviser.setAdviserId(book.getCreatedUser());
		bookAdviser.setChannelId(book.getChannelId());
		bookAdviser.setCreatedUser(book.getCreatedUser());
		bookAdviser.setTempletId(book.getTempletId());
		bookAdviserBiz.create(bookAdviser);
	}

	@Override
	public Boolean checkISBN4App(String isbn) {
		bookSet.checkISBNEx(BookTypeEnum.BOOK.value, isbn);
		return true;
	}
	Long getAgentIdByType(){
		Long checkAgentId = agentConsr.getAgentIdByType(AgentTypeEnum.AGENT_TYPE_WYS.getCode());
		return  checkAgentId;
	}

	/**
	 * 根据渠道ID分页获取图书列表
	 */
	@ParamLog(description = "根据渠道ID分页获取图书列表")
	@Override
	public PageBean getBookBaseInfoListByChannelId4Adviser(Long partyId,Long channelId, PageParam pageParam) throws BizException {
		Map<String,Object> paramMap = Maps.newHashMap();
		paramMap.put("adviserId",partyId);
		paramMap.put("channelId",channelId);
		/*paramMap.put("typeCode",BookTypeEnum.BOOK.value);*/
		PageBean pageBean = bookDao.listPage(pageParam, paramMap, "getListPage4Adviser");
		if (null == pageBean || ListUtils.isEmpty(pageBean.getRecordList())) {
			return new PageBean(0, 0, Lists.newArrayList());
		}else{
			// 设置模板名字
			bookSet.setTemplateName(pageBean.getRecordList());
		}
		return pageBean;
	}

	@Transactional(rollbackFor = Exception.class)
	@ParamLog(description = "根据ID修改时间")
	@Override
	public void updateTimeById(Long bookId) throws BizException {
		if(null == bookId){
			return;
		}
		Map<String,Object> paramMap = Maps.newHashMap();
		paramMap.put("bookId",bookId);
		bookDao.updateTimeById(paramMap);
	}

	@Transactional(rollbackFor = Exception.class)
	@ParamLog(description = "批量根据ID修改时间")
	@Override
	public void updateTimeByIds(BookAdviserUpdateTimeDTO bookAdviserUpdateTimeDTO) throws BizException {
		if(ListUtils.isEmpty(bookAdviserUpdateTimeDTO.getBookIds()) && null == bookAdviserUpdateTimeDTO.getBookId() ){
			return;
		}
		Map<String,Object> paramMap = Maps.newHashMap();
		paramMap.put("adviserId",bookAdviserUpdateTimeDTO.getAdviserId());
		paramMap.put("channelId",bookAdviserUpdateTimeDTO.getChannelId());
		if(null != bookAdviserUpdateTimeDTO.getBookId()){
			paramMap.put("bookId",bookAdviserUpdateTimeDTO.getBookId());
		}else{
			paramMap.put("bookIds",bookAdviserUpdateTimeDTO.getBookIds());
		}
		bookDao.updateTimeByIds(paramMap);
	}
}
